
/*
 *                        classe Soap2D_scene                        
 *                        -------------------                         
 *                                                                    
 * Cette classe code la scne que l'on veut reprsenter.              
 * Une scne est reprsente par :                                    
 *                   - son nom                                        
 *                   - sa taille (angle horizontal et vertical)       
 *                   - l'quation du plan (ax+by+cz=0)                   
 *                                                                    
 * Une scne est compose :                                           
 *                   - d'une camra fixe (pied fixe)                  
 *                   - de plusieurs personnages                       
 *                   - (ventuellement) de points de synchronisation  
 *                                                                    
 */ 


#include "Soap2D_scene.h"


//
// Constructeur
// ------------
// -> Soap2D_scene(string name, float Va, float Vb, float Vc)
//
// Construit une instance d'une scne 2D,  partir de ses spcifications 
//   (ie: son nom, l'quation du plan, et la taille de la scne)
//
Soap2D_scene::Soap2D_scene(string name, float Va, float Vb, float Vc, float Vd, float ouvX, float ouvY)
{
  nom=name;
  eq_plan=new Soap_plan(Va,Vb,Vc,Vd);
  ouverture_x=ouvX;
  ouverture_y=ouvY;
}


//
// Destructeur
// -----------
// -> ~Soap2D_scene()
//
// Dtruit les pointeurs et les vecteurs.
//
Soap2D_scene::~Soap2D_scene()
{
  delete eq_plan;
  delete camera_scene;
  persos.~vector();
  pts_synchro.~vector();
}


//---------------------------------------------------


//
// -> AjoutPerso(string n, float ray,float Vr, float Vg, float Vb)
//
// Permet d'ajouter un personnage  la scne. 
// Pour ajouter un personnage, on donne en paramtre :
//                 - le nom du perso
//                 - la couleur du personnage (valeurs rgb)
//
void Soap2D_scene::AjoutPerso(string n, float ray, float Vr, float Vg, float Vb)
{
  Soap2D_personnage *tmp=new Soap2D_personnage(n,ray,Vr,Vg,Vb);
  persos.push_back(tmp);
}


//
// ->AjoutPtSynchro(string perso1, int numpt1 ,string perso2, int numpt2, int typSync1, int typSync2)
//
// Permet d'ajouter un point de synchronisation (entre 2 personnages, ou entre 1 perso et la camra). 
// Pour ajouter un point de synchronosation, on donne en paramtre :
//                 - le nom du personnage 1 (peut tre aussi la camra)
//                 - le nom du personnage 2
//                 - le numro d'un point de passage du perso1
//                 - le numro d'un point de passage du perso2
//                 - le type de synchronisation pour le perso 1 (si la synchro se fait au dpart ou  l'arrive du point)
//                 - le type de synchronisation pour le perso 2
// Ainsi, on aura un point de synchronisation entre 2 persos, cette synchro se faisant au moment ou ces persos arriveront 
//  (ou partiront) de leur point de passage donne en paramtre. 
//
void Soap2D_scene::AjoutPtSynchro(string perso1, int numpt1 ,string perso2, int numpt2, string typSync1, string typSync2)
{
  bool p1_trouve=false;
  bool p2_trouve=false;
  Soap2D_personnage *p1;
  Soap2D_personnage *p2;
  int i=0;
  int s1=0,s2=0; // type de synchronisation
   
  if (typSync1=="arrive") s1=1;
  if (typSync1=="dpart") s1=2;
  if (typSync2=="arrive") s2=1;
  if (typSync2=="dpart") s2=2;
  while (i<persos.size() || (p1_trouve==false && p2_trouve==false))
    {
      if (persos[i]->nom==perso1)
	{
	  p1=persos[i];
	  p1_trouve=true;
	}
      if (persos[i]->nom==perso2)
	{
	  p2=persos[i];
	  p2_trouve=true;
	}
      i++;
    }
  
  Soap2D_pointsynchro *ps=new Soap2D_pointsynchro(p1,p2,numpt1,numpt2,s1,s2); 
  pts_synchro.push_back(ps);

  
}



//
// -> Camera(float posx, float posy, float ratio_cam)
//
// Permet de spcifier la camra filmant la scne.
// On spcifie cette camra grce  :
//            - sa position (ie: la position de son pied, en quelque sorte)
//            - le ratio de cette camra (si on filme en 16/9, 4/3 ...)
// La position de la camra, aussi bien que son ratio, sont immuables pour une scne donne.
//
void Soap2D_scene::Camera(float posx, float posy, float ratio_cam)
{
  camera_scene=new Soap2D_camera(posx,posy,ratio_cam);
}



void Soap2D_scene::AjoutSynchroCam(string per, int poscam, int pospers,string Tsyncp)
{
  bool per_trouve=false;
  Soap2D_personnage *p;
  int i=0;
  int s1=0; // type de synchronisation
   
  if (Tsyncp=="arrive") s1=1;
  if (Tsyncp=="dpart") s1=2;
  
  while (i<persos.size() || per_trouve==false)
    {
      if (persos[i]->nom==per)
	{
	  p=persos[i];
	  per_trouve=true;
	}
      i++;
    }

  Soap2D_synchrocam *sc=new Soap2D_synchrocam(camera_scene, p,poscam,pospers,s1);
  pts_camsynchro.push_back(sc);
}




void Soap2D_scene::affiche_scene()
{
  cout<<"nom scene : "<<nom<<" plan : "<<eq_plan->a<<", "<<eq_plan->b<<", "<<eq_plan->c<<", "<<eq_plan->d<<endl;
  cout<<"          ouvx : "<<ouverture_x<<endl;
  cout<<"          ouvy : "<<ouverture_y<<endl;
  cout<<"persos : "<<endl;
  for(int i =0;i<persos.size();++i){
          cout<<"\tnom : "<<persos[i]->nom;
          cout<<"\n\ttaille : "<<persos[i]->rayon_sphere_englobante;
          cout<<"\n\tcouleur : "<<persos[i]->couleur->r<<" "<<persos[i]->couleur->g<<" "<<persos[i]->couleur->b;
          cout<<"\n\tpoint_passages:\n";
          for(int j=0;j<persos[i]->pt_passage.size();++j){
                  cout<<"\t\t coord: "<<persos[i]->pt_passage[j]->position->x<<", "<<persos[i]->pt_passage[j]->position->y<<", "<<persos[i]->pt_passage[j]->rayon_disque<<endl;
                  cout<<"\t\t depla: "<<persos[i]->pt_passage[j]->type_deplacement->allure<<", "<<persos[i]->pt_passage[j]->type_deplacement->tps_stop<<", "<<persos[i]->pt_passage[j]->type_deplacement->angle_corps<<", "<<persos[i]->pt_passage[j]->type_deplacement->angle_tete<<endl<<endl;
                  }
                  }
  cout<<"\npoint synchros:\n";
  for(int i=0;i<pts_synchro.size();++i){
           cout<<"\tpt"<<i<<" : "<<pts_synchro[i]->perso1->nom<<" "<<pts_synchro[i]->pos_pt1<<" "<<pts_synchro[i]->type_synchro1<<endl;
           cout<<"\t      "<<pts_synchro[i]->perso2->nom<<" "<<pts_synchro[i]->pos_pt2<<" "<<pts_synchro[i]->type_synchro2<<endl;
           cout<<endl;
           }
  cout<<"\ncamera:\n";
  cout<<"\tratio : "<<camera_scene->ratio<<endl;
  cout<<"\tcoord : "<<camera_scene->position->x<<", "<<camera_scene->position->y<<endl;
  cout<<"\tpoint_cam : "<<endl;
  for(int i=0;i<camera_scene->pt_camera.size();++i){
          cout<<"\t\t"<<camera_scene->pt_camera[i]->ouvX<<" "<<camera_scene->pt_camera[i]->ouvY<<" "<<camera_scene->pt_camera[i]->theta<<" "<<camera_scene->pt_camera[i]->phy<<endl;
  		}
  cout<<"\nsynchro cam : "<<endl;
	  for(int i=0;i<pts_camsynchro.size();++i){
          cout<<"\t\t"<<"perso="<<pts_camsynchro[i]->perso->nom<<" poscam="<<pts_camsynchro[i]->pos_cam<<" pospers="<<pts_camsynchro[i]->pos_perso<<" typsynchro= "<<pts_camsynchro[i]->type_synchro_perso<<endl;
  	}
 cout<<endl<<endl;
          
}
