#define SOAP2D_SCENE_H

/*
 *                        classe Soap2D_scene                         
 *                        -------------------                         
 *                                                                    
 * Cette classe code la scne que l'on veut reprsenter.              
 * Une scne est reprsente par :                                    
 *                   - son nom                                        
 *                   - sa taille (angle horizontal et vertical)       
 *                   - l'quation du plan (ax+by+cz=0)                
 *                                                                    
 * Une scne est compose :                                           
 *                   - d'une camra fixe (pied fixe)                  
 *                   - de plusieurs personnages                       
 *                   - (ventuellement) de points de synchronisation  
 *                                                                    
 */ 
	
#include <string>	
#include <vector>
#include <iostream>
#ifndef SOAP_PLAN_H
#include "../SoapGeneral/Soap_plan.h"
#endif
#ifndef SOAP2D_PERSONNAGE_H
#include "Soap2D_personnage.h"
#endif
#ifndef SOAP2D_POINTSYNCHRO_H
#include "Soap2D_pointsynchro.h"
#endif
#ifndef SOAP_VARIABLE_H
#include "../SoapGeneral/Soap_variables.h"
#endif
#ifndef SOAP2D_CAMERA_H
#include "Soap2D_camera.h"
#endif
#ifndef SOAP2D_SYNACHROCAM_H
#include "Soap2D_synchrocam.h"
#endif
using namespace std;

class Soap2D_scene
{
 public:
  
  /* ATTRIBUTS */
  string nom;                     // nom de la scne
  Soap_plan *eq_plan;             // quation du plan de la scne (de type ax+by+cz+d=0) 
  float ouverture_x;              // tendue en horizontal de la scne entire
  float ouverture_y;              // tendue en vertical de la scne entire
  Soap2D_camera *camera_scene;    // camra filmant la scne

  vector<Soap2D_personnage*> persos;        // liste des personnages prsents dans la scne
  vector<Soap2D_pointsynchro*> pts_synchro; // liste des points de synchronisation
  vector<Soap2D_synchrocam*> pts_camsynchro; 


  /* METHODES */

  // constructeur, destructeur
  Soap2D_scene(string name, float Va, float Vb, float Vc, float Vd,float ouvX, float ouvY);
  ~Soap2D_scene();

  // ajout d'un personnage  la scne
  void AjoutPerso(string n, float ray,float Vr, float Vg, float Vb);
  // spcification de la camra
  void Camera(float posx, float posy, float ratio_cam);
  // ajout d'un point de synchronisation entre 2 personnages
  void AjoutPtSynchro(string perso1, int numpt1 ,string perso2, int numpt2, string typSync1, string typSync2);
  // ajout d'un point de synchronisation pour la camra
  void AjoutSynchroCam(string per, int poscam, int pospers,string Tsyncp);
  //affichage pour le debuggage
  void affiche_scene();
};

