
/*
 *                        classe Interpolation_NURBS
 *                        -------------------                         
 *                                                                    
 * Cette classe permet de raliser les interpolations  l'aide de nurbs.
 * Elle hrite de Interpolation
 * Interpolation_NURBS est reprsente par :                                    
 *                   - la nurbs
 *                   - un tableau de correspondance entre lea parametre u et la distance sur la courbe
 *                                                                    
 */ 

#include "Interpolation_NURBS.h"


//
// Constructeur
// ------------
// -> Interpolation_NURBS()
//
// Construit une instance d'une interpolation_NURBS
//
Interpolation_NURBS::Interpolation_NURBS(){};

// -> Interpolation_NURBS(Vector_Point3Df pCles)
//
// Construit une instance d'une interpolation_NURBS  partir de la liste de points cles
//
Interpolation_NURBS::Interpolation_NURBS(Vector_Point3Df pCles):Interpolation(){//par dfaut la dimention est de 3
 
  pointsCles=pCles;
  dimension=3;
 
  courbe.globalInterp(pointsCles,dimension);

  /* la mthode globalInterp construits une nurbs passant exactement
  /*   par tous les points 3D du vecteur spcifi
  /*							               */
  Point3Df pt1 = Cp(0.f,courbe); //point initial
  Point3Df pt2;
  double dist=0.f;
  tableau.push_back(dist);
                
  //construction du tableau de correspondance
  for(double u = pas;u <= 1.0 +pas ;u+=pas) {
      pt2 = Cp(u,courbe);
      dist += distancePoint(pt1,pt2);//distance linaire entre deux points trs proche
      tableau.push_back(dist);
      pt1=pt2; 
     }
}
     

//---------------------------------------------------


//
// -> distancePoint(Point3Df pt1 , Point3Df pt2)
//
// Obtenir la distance linraire entre deux points
//                 - le point1
//                 - le point2
//   <-la distance calcule


double Interpolation_NURBS::distancePoint(Point3Df pt1 , Point3Df pt2)
 {
  return (double)sqrt((pt2.x() - pt1.x())*(pt2.x() - pt1.x()) + (pt2.y()- pt1.y())* (pt2.y()- pt1.y())+ (pt2.z() - pt1.z())*(pt2.z() - pt1.z()));

 }
 
 
//
// -> distanceRelative(Point3Df pt1 , Point3Df pt2)
//
// Obtenir la distance relative entre deux points
//                 - le point1
//                 - le point2
//   <-la distance calcule

 double distanceRelative(Point3Df pt1 , Point3Df pt2)
 {
  return (double)(pt2.x() - pt1.x()) + (pt2.y()- pt1.y()) + (pt2.z() - pt1.z());

 }
 
 
//
// -> distance_vs_u(double dist)
//
// Obtenir le parametre u suivant la distance
//                 - la distance
// <- valeur de u

double Interpolation_NURBS::distance_vs_u(double dist) {
   double res,sup,inf;
   int i=0;
   while((i<tableau.size())&&(dist>tableau[i])) i++;
   sup = tableau[i];
   inf = tableau[i-1];
   double val=sup-inf;
   
   if(sup<=inf){res=0;}
   else{
   res = (i-1)*pas +(dist-inf)*pas /(sup-inf);//interpolation linaire pour calculer la valeur de u
   

   }
   return res;
 }
 
 
  
//
// -> curveDistance(double dist)
//
// Obtenir le point3D correspondant  la distance curviligne
//                 - la distance
// <- point3D

Point3Df Interpolation_NURBS::curveDistance(double dist) {
   double u = distance_vs_u(dist);
   return Cp(u,courbe);//mthode de la librairie nurbs++
}

//
// -> u_pointplusproche(Point3Df point)
//
// Obtenir le parametre u du Point3Df le plus proche de point
// On ralise un balayage sur le parametre u et on garde la valeur de u
// correspondant  la distance minimal entre le point et le point paramtr par u
//                 - le point
// <- valeur de u

double Interpolation_NURBS::u_pointplusproche(Point3Df point){

double distance_min=-1.0;
double distance=0.0;
Point3Df pt;
double val_u=0.0;

for(double u =0.0;u <= 1.0 +pas ;u+=pas) {//balayage des u

   pt = Cp(u,courbe);//cration d'un point paramtr par u
   distance = distancePoint(pt, point);//calcul de la distance
   if((distance<=distance_min)||(distance_min==-1.0)) {
   val_u=u;//on garde la valeur de u du point le plus proche de point
   distance_min=distance;
   }     

 }
return val_u;
}

//
// -> distance_upt1vsupt2(double upt1, double upt2)
//
// Obtenir la distance curviligne entre deux points paramtrs par leur valeur de u
// On utilise pour cela le tableau de corrrespondance
//                 - la valeur de u du point 1
//                 - la valeur de u du point 2
// <- distance calcule

double Interpolation_NURBS::distance_upt1vsupt2(double upt1, double upt2){
  int indice1 = (int)(upt1/pas);//approximation du u
  int indice2 = (int)(upt2/pas);

  double res;

  if(indice1<=indice2) res=tableau[indice2] - tableau[indice1];
  else res=tableau[indice1]-tableau[indice2];
  return res;
}

//
// -> distance_pt1vspt2(Point3Df pt1, Point3Df pt2)
//
// Obtenir la distance curviligne entre deux points
//                 - le point 1
//                 - le point 2
// <- distance calcule

double Interpolation_NURBS::distance_pt1vspt2(Point3Df pt1, Point3Df pt2){
   double upt1=u_pointplusproche(pt1);//recherche de la valeur de u du point pt1
   double upt2=u_pointplusproche(pt2);//recherche de la valeur de u du point pt2
   
   return distance_upt1vsupt2(upt1,upt2);

  }
  
  
//
// -> min_ordonnee()
//
// Obtenir la valeur de l'ordonne minimum des pointsCles
// <- ordonne

  
double Interpolation_NURBS::min_ordonnee(){
  double res=-1.0;
  for(int i=0;i<pointsCles.size();i++){
   if((pointsCles[i].y()<res)||(res==-1.0)) res=pointsCles[i].y(); 
  }
  return res;
}

//
// -> min_abscisse()
//
// Obtenir la valeur de l'abscisse minimum des pointsCles
// <- abscisse

double Interpolation_NURBS::min_abscisse(){
  double res=-1.0;
  for(int i=0;i<pointsCles.size();i++){
   if((pointsCles[i].x()<res)||(res==-1.0)) res=pointsCles[i].x(); 
  }
  return res;
}

//
// -> max_ordonnee()
//
// Obtenir la valeur de l'ordonne maximum des pointsCles
// <- ordonne

double Interpolation_NURBS::max_ordonnee(){
  double res=-1.00001;
  for(int i=0;i<pointsCles.size();i++){
   if((pointsCles[i].y()>res)||(res==-1.00001)) res=pointsCles[i].y(); 
  }
  return res;
}

//
// -> max_abscisse()
//
// Obtenir la valeur de l'abscisse maximum des pointsCles
// <- abscisse
double Interpolation_NURBS::max_abscisse(){
  double res=-1.0;
  for(int i=0;i<pointsCles.size();i++){
   if((pointsCles[i].x()>res)||(res==-1.0)) res=pointsCles[i].x(); 
  }
  return res;
}


//
// ->obtient_ordonnee(double absc)
//
// Obtenir la valeur de l'ordonne suivant l'abscisse
// On ralise un balayage ligne en recherchant  chaque ligne la distance minimale entre
// le point paramtr par l'abscisse et le point paramtr par u.
// On retourne au final l'ordonne du point d'intersection entre la nurbs et la droite x=absc
//                 - l'abscisse du point
// <- ordonne

double Interpolation_NURBS::obtient_ordonnee(double absc){

double distance_min=-1.00001,dist;
double u_point,val_u;
double pas_y=0.2;
double ymin=min_ordonnee();//rcupration de l'ordonne minimale
double ymax=max_ordonnee();//rcupration de l'ordonne maximale

for(double y=ymin; y<=ymax;y+=pas_y){//balayage ligne entre ymin et ymax

   Point3Df pt(absc,y,0);//cration du point 
   val_u=u_pointplusproche(pt);
   dist=distancePoint(pt,Cp(val_u,courbe));
   if((dist<distance_min)||(distance_min==-1.00001)) {
   u_point=val_u;
   distance_min=dist;//distance minimale entre le point paramtr par l'abscisse et le point paramtr par u
   }    
   }

   
   return (Cp(u_point,courbe)).y();
   //ordonne du point d'intersection entre la nurbs et la droite x=absc

}

//
// ->obtient_x_vs_z(double z1)
//
// Obtenir la valeur de l'abscisse suivant la cote
// On ralise un balayage suivant x en recherchant la distance minimale entre
// le point paramtr par la cote et le point paramtr par u.
// On retourne au final l'abscisse du point d'intersection entre la nurbs et la droite z=z1
//                 - la cote du point
// <- abscisse

double Interpolation_NURBS::obtient_x_vs_z(double z){

double distance_min=-1.00001,dist;
double u_point,val_u;
double pas_x=0.2;
double xmin=min_abscisse();//rcupration de l'abscisse minimale
double xmax=max_abscisse();//rcupration de l'abscisse maximale

for(double x=xmin; x<=xmax;x+=pas_x){//balayage entre xmin et xmax
   Point3Df pt(x,0.0,z);
   val_u=u_pointplusproche(pt);
   dist=distancePoint(pt,Cp(val_u,courbe));
   if((dist<distance_min)||(distance_min==-1.00001)) {
   u_point=val_u;
   distance_min=dist;//distance minimale entre le point paramtr par la cote et le point paramtr par u
   }    
   }

   return (Cp(u_point,courbe)).x();
	//abscisse du point d'intersection entre la nurbs et la droite z=z1
}

//
// ->:obtient_y_vs_z(double z1)
//
// Obtenir la valeur de l'ordonne suivant la cote
// On ralise un balayage suivant y en recherchant  chaque ligne la distance minimale entre
// le point paramtr par la cote et le point paramtr par u.
// On retourne au final l'abscisse du point d'intersection entre la nurbs et la droite z=z1
//                 - la cote du point
// <- ordonne

double Interpolation_NURBS::obtient_y_vs_z(double z){

double distance_min=-1.00001,dist;
double u_point,val_u;
double pas_y=0.2;
double ymin=min_ordonnee();//rcupration de l'ordonne minimale
double ymax=max_ordonnee();//rcupration de l'ordonne maximale

for(double y=ymin; y<=ymax;y+=pas_y){
   Point3Df pt(0.0,y,z);
   val_u=u_pointplusproche(pt);
   dist=distancePoint(pt,Cp(val_u,courbe));
   if((dist<distance_min)||(distance_min==-1.00001)) {
   u_point=val_u;
   distance_min=dist;//distance minimale entre le point paramtr par la cote et le point paramtr par u
   }    
   }

   return (Cp(u_point,courbe)).y();
    //ordonne du point d'intersection entre la nurbs et la droite z=z1

}

//
// ->:obtient_u_vs_absisse(double absc)
//
// Obtenir la valeur de u suivant l'abscisse
// On ralise un balayage suivant y en recherchant  chaque ligne la distance minimale entre
// le point paramtr par la cote et le point paramtr par u.
// On retourne la valeur de u du point d'intersection entre la nurbs et la droite z=z1
//                 - l'abscisse
// <- la valeur de u

double Interpolation_NURBS::obtient_u_vs_absisse(double absc){

double distance_min=-1.00001,dist;
double u_point,val_u;
double pas_y=0.2;
double ymin=min_ordonnee();//rcupration de l'ordonne minimale
double ymax=max_ordonnee();//rcupration de l'ordonne maximale

for(double y=ymin; y<=ymax;y+=pas_y){
   Point3Df pt(absc,y,0);
   val_u=u_pointplusproche(pt);
   dist=distancePoint(pt,Cp(val_u,courbe));
   if((dist<distance_min)||(distance_min==-1.00001)) {u_point=val_u;distance_min=dist;}    
   }

   return u_point;

}




