
/*
 *                        classe Soap3D_scene                        
 *                        -------------------                         
 *                                                                    
 * Cette classe code la scne que l'on veut reprsenter.              
 * Une scne est reprsente par :                                    
 *                   - son nom                                        
 *                   - sa taille (angle horizontal et vertical)       
 *                   - l'quation du plan (ax+by+cz=0)                   
 *                                                                    
 * Une scne est compose :                                           
 *                   - d'une camra fixe (pied fixe)                  
 *                   - de plusieurs personnages                    
 *                                                                    
 */ 

#include "Soap3D_scene.h"



//
// Constructeur
// ------------
// -> Soap3D_scene(Soap2D_scene *sc2D)
//
// Construit une instance d'une scne 3D,  partir de la mme scne en 2D 
//
Soap3D_scene::Soap3D_scene(Soap2D_scene *sc2D)
{
  Soap_point2D *p2d;
  Soap_point3D *p3d;

  // on recopie les informations en commun (nom, plan ...)
  cout<<"plan"<<endl;
  nom=sc2D->nom;
  eq_plan=new Soap_plan(sc2D->eq_plan->a,sc2D->eq_plan->b,sc2D->eq_plan->c,sc2D->eq_plan->d);
  ouverture_x=sc2D->ouverture_x;
  ouverture_y=sc2D->ouverture_y;

  // on recopie les informations pour la camra
  cout<<"camera"<<endl;
  //p2d=new Soap_point2D(sc2D->camera_scene->position->x, sc2D->camera_scene->position->y);
  p3d=new Soap_point3D(0,0,0);
  Camera(p3d,sc2D->camera_scene->ratio);
  cout<<"camera fixe"<<endl;
  // points de passage de la camra
  Soap2D_pointcamera *pc;
  for (int i=0; i<sc2D->camera_scene->pt_camera.size();i++)
  {
    pc=sc2D->camera_scene->pt_camera[i];
    camera_scene->AjoutPointCam(pc->theta, pc->phy, pc->ouvX, pc->ouvY);
    cout<<"point camera"<<endl;
  }

  //on recopie les informations pour les personnages
  cout<<"persos"<<endl;
  Soap2D_personnage *perso;
  Soap2D_pointpassage *ptp;
  for (int i=0; i<sc2D->persos.size();i++)
  {
    perso=sc2D->persos[i];
    AjoutPerso(perso->nom,perso->rayon_sphere_englobante,perso->couleur->r,perso->couleur->g,perso->couleur->b);
    for (int j=0; j<perso->pt_passage.size();j++)
    {
       ptp=perso->pt_passage[j];
       p2d=new Soap_point2D(ptp->position->x,ptp->position->y);
       p3d=p2d->Enpoint3D(sc2D->ouverture_x, sc2D->ouverture_y, perso->rayon_sphere_englobante, ptp->rayon_disque);
       Soap_deplacement *dep=ptp->type_deplacement;
       persos[i]->NouveauPointPassage(p3d,dep->angle_tete,dep->angle_corps,dep->allure,dep->tps_stop);
    }
  }
}


//
// Destructeur
// -----------
// -> ~Soap3D_scene()
//
// Dtruit les pointeurs et les vecteurs.
//
Soap3D_scene::~Soap3D_scene()
{
  delete eq_plan;
  delete camera_scene;
  persos.~vector();
}



//---------------------------------------------------


//
// -> AjoutPerso(string n, float ray,float Vr, float Vg, float Vb)
//
// Permet d'ajouter un personnage  la scne. 
// Pour ajouter un personnage, on donne en paramtre :
//                 - le nom du perso
//                 - la couleur du personnage (valeurs rgb)
//
void Soap3D_scene::AjoutPerso(string n, float ray, float Vr, float Vg, float Vb)
{
  Soap3D_personnage *tmp=new Soap3D_personnage(n,ray,Vr,Vg,Vb);
  cout<<"ajout perso : "<<tmp->nom<<endl;
  persos.push_back(tmp);
}


//
// -> Camera(Soap_point3D *p3d,float ratio_cam)
//
// Permet de spcifier la camra filmant la scne.
// On spcifie cette camra grce  :
//            - sa position (ie: la position de son pied, en quelque sorte)
//            - le ratio de cette camra (si on filme en 16/9, 4/3 ...)
// La position de la camra, aussi bien que son ratio, sont immuables pour une scne donne.
//
void Soap3D_scene::Camera(Soap_point3D *p3d,float ratio_cam)
{
  camera_scene=new Soap3D_camera(p3d,ratio_cam);
}

void Soap3D_scene::affiche_scene()
{
  cout<<"nom scene : "<<nom<<" plan : "<<eq_plan->a<<", "<<eq_plan->b<<", "<<eq_plan->c<<", "<<eq_plan->d<<endl;
  cout<<"          ouvx : "<<ouverture_x<<endl;
  cout<<"          ouvy : "<<ouverture_y<<endl;
  cout<<"persos : "<<endl;
  for(int i =0;i<persos.size();++i){
          cout<<"\tnom : "<<persos[i]->nom;
          cout<<"\n\ttaille : "<<persos[i]->rayon_sphere_englobante;
          cout<<"\n\tcouleur : "<<persos[i]->couleur->r<<" "<<persos[i]->couleur->g<<" "<<persos[i]->couleur->b;
          cout<<"\n\tpoint_passages:\n";
          for(int j=0;j<persos[i]->pt_passage.size();++j){
                  cout<<"\t\t coord: "<<persos[i]->pt_passage[j]->position->x<<", "<<persos[i]->pt_passage[j]->position->y<<", "<<persos[i]->pt_passage[j]->position->z<<endl;
                  cout<<"\t\t deplacement: "<<persos[i]->pt_passage[j]->allure<<", "<<persos[i]->pt_passage[j]->angle_corps<<", "<<persos[i]->pt_passage[j]->angle_tete<<endl<<endl;
                  }
                  }
       
          cout<<"\ncamera:\n";
          cout<<"\tratio : "<<camera_scene->ratio<<endl;
          cout<<"\tcoord : "<<camera_scene->position->x<<", "<<camera_scene->position->y<<endl;
          cout<<"\tpoint_cam : "<<endl;
          for(int i=0;i<camera_scene->pt_camera.size();++i){
          cout<<"\t\t"<<camera_scene->pt_camera[i]->ouvX<<" "<<camera_scene->pt_camera[i]->ouvY<<" "<<camera_scene->pt_camera[i]->theta<<" "<<camera_scene->pt_camera[i]->phy<<endl;
}
          cout<<endl;
          
}
