
/*
 *                        classe Soap_interpolation
 *                        -------------------                         
 *                                                                    
 * Cette classe permet de raliser l'ensemble des interpolations demandes.
 * Elle agit en ce sens comme une boite noire pour les autres classes de Soap
 * qui lui font appel
 * Une Soap_interpolation est compose :                                    
 *                   - d'un vector de trajectoires qui contient pour chaque personnage
 *			3D, les informations sur son dplacement et sa vitesse                                    
 *                   - d'une Soap_ralisation qui contient les informations relatives  la camera
 *			(angles de vues, zoom)
 *
 */ 


#include"Soap_interpolation.h"      


//
// Constructeur
// ------------
// -> Soap_interpolation(vector<Soap3D_personnage *> persos, Soap3D_camera *camera)
//
// Construit une instance d'une Soap_interpolation,  partir de la liste de personnage et de la camra 
//

Soap_interpolation::Soap_interpolation(vector<Soap3D_personnage *> persos, Soap3D_camera *camera){
                               
     for(int i = 0;i<persos.size();i++){
         Soap_trajectoire *tmp=new Soap_trajectoire(persos[i]);//association d'une trajectoire  chaque perosnnage
         trajectoires.push_back(tmp);
        }         
	

    realisation = new Soap_realisation(camera);//cration de la ralisation de la camra
                                                                                                       
};


//
// -> recherchePerso(string nomp)
//
// Recherche l'indice du personnage dans la liste de personnage suivant le nom
//            - le nom du personnage
//  <-l'indice du personnage
//
     
int Soap_interpolation::recherchePerso(string nomp){
                  
                  int i = 0;
                  bool trouve =false;
                  while((i<trajectoires.size())&& (!trouve)){//parcours de la liste des trajectoires
                    if(trajectoires[i]->perso->nom == nomp)
                       trouve = true;//le personnage est identifi
                     else                      
                       i++;        
                   }
                   if(!trouve) return -1;//on retourne -1 si le personnage est absent de la liste
                               else return i;
}; 


//
// -> position(string nom_perso, float temps)
//
// Obtenir la position d'un personnage suivant le temps
//            - le nom du personnage
//            - le temps parametre
//  <- le point 3D de la position
//

Soap_point3D Soap_interpolation::position(string nom_perso, float temps){
		Soap_point3D res(0,0,0);
                int indice = recherchePerso(nom_perso);//rcupration de l'indide du personnage
                if(indice==-1) {cout<<endl<<nom_perso<<" n existe pas "<<endl;exit(-1);}
			//sinon le personnage est bien identifi
                        else res = trajectoires[indice]->position(temps);//la position est calcule dans la trajectoire
			
			return res;
                      };


//
// -> position(string nom_perso, float temps)
//
// Obtenir le temps entre deux points
//            - le nom du personnage
//            - l'indice du point 1
//            - l'indice du point 2
//  <- le temps calcul
//

float Soap_interpolation::temps_entre(string nom_perso, int p1, int p2){
  
  float res=0.0;
  int indice = recherchePerso(nom_perso); //rcupration de l'indide du personnage
   if(indice==-1) {cout<<endl<<nom_perso<<" n existe pas "<<endl;exit(-1);}
      else{ //sinon le personnage est bien identifi
        res=trajectoires[indice]->temps_entre(p1,p2);//le temps entre deux points est calcul dans la trajectoire
       }

   return res;
  };

//
// ->angle_corps(string nom_perso, float temps)
//
// Obtenir l'angle du corps d'un personnage fonction du temps
//            - le nom du personnage
//            - le temps du film 
//  <- l'angle du corps
//

float Soap_interpolation::angle_corps(string nom_perso, float temps){
  float res=0.0;
  int indice = recherchePerso(nom_perso); //rcupration de l'indide du personnage
   if(indice==-1) {cout<<endl<<nom_perso<<" n existe pas "<<endl;exit(-1);}
      else{  //sinon le personnage est bien identifi
        res=trajectoires[indice]->angle_corps(temps);//l'angle du corps en fonction du temps est calcul dans la trajectoire
       }

   return res;
};

//
// ->angle_tete(string nom_perso, float temps)
//
// Obtenir l'angle de la tete d'un personnage fonction du temps
//            - le nom du personnage
//            - le temps du film 
//  <- l'angle de la tete
//

float Soap_interpolation::angle_tete(string nom_perso, float temps){
  float res=0.0;
  int indice = recherchePerso(nom_perso); //rcupration de l'indide du personnage
   if(indice==-1) {cout<<endl<<nom_perso<<" n existe pas "<<endl;exit(-1);}
      else{ //sinon le personnage est bien identifi
        res=trajectoires[indice]->angle_tete(temps);//l'angle de la tete en fonction du temps est calcul dans la trajectoire
       }

   return res;
}; 
  
//
// ->creerInterpolation(string nom_perso)
//
//Permet de creer les interpolations sur la vitesse, les angles de deplacement d'un personnage
//et sur les angles de vues et le zoom de la camra
//Ces interpolations ne sont pas crer  l'appel du constructeur car elles dpendent galement
//des temps de synchronisation
//            - le nom du personnage
//

void Soap_interpolation::creerInterpolation(string nom_perso){
  
   int indice = recherchePerso(nom_perso); //rcupration de l'indide du personnage
   if(indice==-1) {cout<<endl<<nom_perso<<" n existe pas "<<endl;exit(-1);}
      else{  //sinon le personnage est bien identifi
        trajectoires[indice]->creerNurbsAngleCorps();
	trajectoires[indice]->creerNurbsAngleTete();
       }
       realisation->creerNurbsRealisation();
  }
  
//
// ->creerInterpolationPerso(string nom_perso)
//
//Permet de creer les interpolations sur les angles de deplacement d'un personnage
//
//            - le nom du personnage
//
void Soap_interpolation::creerInterpolationPerso(string nom_perso){
  
   int indice = recherchePerso(nom_perso); //rcupration de l'indide du personnage
   if(indice==-1) {cout<<endl<<nom_perso<<" n existe pas "<<endl;exit(-1);}
      else{  //sinon le personnage est bien identifi
        trajectoires[indice]->creerNurbsAngleCorps();
	trajectoires[indice]->creerNurbsAngleTete();
       }

  }
  
//
// ->creerInterpolationRealisation()
//
//Permet de creer les interpolations sur les angles de vues et le zoom de la camra
//
void Soap_interpolation::creerInterpolationRealisation(){
  
   realisation->creerNurbsRealisation();
  }
  
//
// ->imprimeTrajectoire()
//
//Impression de la courbe de la trajectoire au format postscript
//

void Soap_interpolation::imprimeTrajectoire(string nom_perso){
int indice = recherchePerso(nom_perso); //rcupration de l'indide du personnage
   if(indice==-1) {cout<<endl<<nom_perso<<" n existe pas "<<endl;exit(-1);}
      else{  //sinon le personnage est bien identifi

	trajectoires[indice]->getCourbe().writePS("trajectoire.ps",false,1,2);
    }
}

//
// ->imprimeInterpolationPerso(string nom_perso)
//
//Impression des courbes de la vitesse, des angles du corp et de la tete
//

void Soap_interpolation::imprimeInterpolationPerso(string nom_perso){
int indice = recherchePerso(nom_perso); //rcupration de l'indide du personnage
   if(indice==-1) {cout<<endl<<nom_perso<<" n existe pas "<<endl;exit(-1);}
      else{  //sinon le personnage est bien identifi
	bool cp=false;
	int magf=1;
	int dash=2;
      	trajectoires[indice]->getCourbeVitesse().getCourbe().writePSp("vitesse.ps",trajectoires[indice]->getCourbeVitesse().getPointsCles(),cp,magf,dash);
	trajectoires[indice]->getCourbeAngleCorps().getCourbe().writePS("angle_corps.ps",cp,magf,dash);
	trajectoires[indice]->getCourbeAngleTete().getCourbe().writePS("angle_tete.ps",cp,magf,dash);
    }
}



//
// ->imprimeInterpolationCamera()
//
//Impression de l'ensemble des nurbs relatives  la camra
//


void Soap_interpolation::imprimeInterpolationRealisation(){

 realisation->getAngleTheta().getCourbe().writePS("camera_theta.ps",false,1,2);
 realisation->getAnglePhy().getCourbe().writePS("camera_phy.ps",false,1,2);
 realisation->getZoom().getCourbe().writePS("camera_zoom.ps",false,1,2);

}


//
// ->camera_theta(float tps);
//
// Obtenir l'angle theta fonction du temps
//

float Soap_interpolation::camera_theta(float tps){

return realisation->camera_theta(tps);
}
  
//
// ->camera_phy(float tps);
//
// Obtenir l'angle phy de la camra en fonction du temps 
//


  //angle phy de la camra en fonction du temps 
float Soap_interpolation::camera_phy(float tps){

 return realisation->camera_phy(tps);
}
  
//
// ->camera_zoom(float tps);
//
// Obtenir l'ouverture sur les x  de la camra en fonction du temps 
//

float Soap_interpolation::camera_zoom(float tps){

 return realisation->camera_zoom(tps);

}





