#define SOAP_INTERPOLATION_H

/*
 *                        classe Soap_interpolation
 *                        -------------------                         
 *                                                                    
 * Cette classe permet de raliser l'ensemble des interpolations demandes.
 * Elle agit en ce sens comme une boite noire pour les autres classes de Soap
 * qui lui font appel
 * Une Soap_interpolation est compose :                                    
 *                   - d'un vector de trajectoires qui contient pour chaque personnage
 *			3D, les informations sur son dplacement et sa vitesse                                    
 *                   - d'une Soap_ralisation qui contient les informations relatives  la camera
 *			(angles de vues, zoom)
 *
 */ 

#ifndef SOAP_TRAJECTOIRE_H
#include"Soap_trajectoire.h"
#endif
#include"Soap_realisation.h"
#ifndef SOAP3D_PERSONNAGE_H
#include"Soap3D_personnage.h"
#endif
#ifndef SOAP3D_POINTPASSAGE_H
#include"Soap3D_pointpassage.h"
#endif
#ifndef SOAP3D_SCENE_H
#include "Soap3D_scene.h"
#endif
#ifndef SOAP_POINT3D_H
#include "../SoapGeneral/Soap_point3D.h"
#endif

class Soap_interpolation {
      
public : 
      
                /* ATTRIBUTS */

  vector<Soap_trajectoire *> trajectoires;//liste de trajectoires (une trajectoire par personnage)
     
  Soap_realisation * realisation;//informations sur la camra
  
  
                /* METHODES */

  // constructeur, destructeur  
  Soap_interpolation(vector<Soap3D_personnage *> persos, Soap3D_camera * cam);
  
  //recherche un personnage 3D en identfiant son nom
  int recherchePerso(string nomp);

  //temps pour un personnage entre deux points
  float temps_entre(string nom_perso, int p1, int p2);

  //position 3D d'un personnage en fonction du temps
  Soap_point3D position(string nom_perso, float temps);
  
  //angle du corps d'un personnage en fonction du temps
  float angle_corps(string nom_perso, float temps);
  
  //angle de la tete d'un personnage en fonction du temps
  float angle_tete(string nom_perso, float temps);

  //permet de creer les interpolations sur les angles de deplacement d'un personnage
  //et sur les angles de vues et le zoom de la camra
  void creerInterpolation(string nom_perso);
  
  //permet de creer uniquement les interpolations relatives a la ralisation
  void creerInterpolationRealisation();
  
  //permet de creer uniquement les interpolations sur les angles de deplacement d'un personnage
  void creerInterpolationPerso(string nom_perso);
  
  //imprime la trajectoire du personnage au format postscript
  void imprimeTrajectoire(string nom_perso);
  
  //imprime l'ensemble des nurbs des interpolations d'un personnage (vitesse et angles du corps et de la tete)
  void imprimeInterpolationPerso(string nom_perso);
  
  //imprime l'ensemble des nurbs des interpolations de la realisation(angles de vue et zoom)
  void imprimeInterpolationRealisation();

  //angle theta de la camra en fonction du temps
  float camera_theta(float tps);
  
  //angle phy de la camra en fonction du temps 
  float camera_phy(float tps);
  
  //ouverture sur les x de la camera
  float camera_zoom(float tps);
  

};         
 



