/*
 *                        classe Soap_realisation
 *                        -------------------                         
 *                                                                    
 * C'est dans classe que sont concentres les informations relatives aux changements
 * d'angles de vue et de zoom de la camra au court du temps
 * Une Soap_realisation est compose :                                    
 *                   - d'une camra
 *                   - de deux  interpolations sur les angles de vues (theta et phy)
 *                   - d'une interpolation sur le zoom
 *
 *
 */ 
#include"Soap_realisation.h"

//
// Constructeur
// ------------
// -> Soap_realisation(Soap3D_camera * cam)
//
// Construit une instance d'une Soap_realisation,  partir de la camra
//
Soap_realisation::Soap_realisation(Soap3D_camera * cam){

	camera=cam;

}

//
// -> creerNurbsAngleVue()
//
// Cration de la nurbs des l'angle de vue suivant le temps pour theta et phy
// en chaque points de passage de la camra
//

void Soap_realisation::creerNurbsAnglesVue(){
  
 Vector_Point3Df pts_theta(camera->pt_camera.size());//tableau des points cles de theta
 Vector_Point3Df pts_phy(camera->pt_camera.size());//tableau des points cles de phy
 float tps;
 
 for(int i=0;i<camera->pt_camera.size();i++){
  tps=camera->pt_camera[i]->tps_film;//rcupration du temps du film
  pts_theta[i]=Point3Df(tps,camera->pt_camera[i]->theta,0.0);
  pts_phy[i]=Point3Df(tps,camera->pt_camera[i]->phy,0.0);
 }
 

 angle_theta= Interpolation_NURBS(pts_theta);
 angle_phy= Interpolation_NURBS(pts_phy);
 
}

//
// -> creerNurbsZoom()
//
// Cration de la nurbs du zoom en suivant le temps et l'ouverture sur les x
// en chaque points de passage de la camra
//

void Soap_realisation::creerNurbsZoom(){
  
 Vector_Point3Df pts(camera->pt_camera.size());//tableau des points cles
 float tps;
 
 for(int i=0;i<camera->pt_camera.size();i++){
  tps=camera->pt_camera[i]->tps_film;//rcupration du temps du film
  pts[i]=Point3Df(tps,camera->pt_camera[i]->ouvX,0.0);
 
 }
 

 zoom= Interpolation_NURBS(pts);
 
}

//
// -> creerNurbsRealisation()
//
// Cration des nurbs AnglesVue et Zoom

void Soap_realisation::creerNurbsRealisation(){

creerNurbsAnglesVue();
creerNurbsZoom();

}

//
// -> camera_zoom(float tps)
//
// Obtenir le zoom suivant le temps
//            - le temps
//   <- l'ouverture sur les x
//

float Soap_realisation::camera_zoom(float tps){
 
 return zoom.obtient_ordonnee(tps);
 //il suffit de calculer l'ordonne de la nurbs 2D (temps,ouverturex)

}

//
// -> camera_theta(float tps)
//
// Obtenir l'angle theta suivant le temps
//            - le temps
//   <- l'angle calcul
//


float Soap_realisation::camera_theta(float tps){

  return angle_theta.obtient_ordonnee(tps);

}
//
// -> camera_phy(float tps)
//
// Obtenir l'angle phy suivant le temps
//            - le temps
//   <- l'angle calcul
//


float Soap_realisation::camera_phy(float tps){

  return angle_phy.obtient_ordonnee(tps);

}


