#define SOAP_REALISATION_H

/*
 *                        classe Soap_realisation
 *                        -------------------                         
 *                                                                    
 * C'est dans classe que sont concentres les informations relatives aux changements
 * d'angles de vue et de zoom de la camra au court du temps
 * Une Soap_realisation est compose :                                    
 *                   - d'une camra
 *                   - deux interpolations sur les angles de vues (theta et phy)
 *                   - d'une interpolation sur le zoom
 *
 *
 */ 
#ifndef INTERPOLATION_NURBS_H
#include "Interpolation/Interpolation_NURBS.h"
#endif
#ifndef SOAP_VARIABLES_H
#include "../SoapGeneral/Soap_variables.h"
#endif
#ifndef SOAP3D_CAMERA_H
#include "Soap3D_camera.h"
#endif
#include <vector>


class Soap_realisation {
      
public :

                /* ATTRIBUTS */

  Soap3D_camera * camera;//pointeur vers la camra

  Interpolation_NURBS  angle_theta;//interpolation sur l'angle theta

  Interpolation_NURBS  angle_phy;//interpolation sur l'angle phy
  
  Interpolation_NURBS zoom;//interpolation sur le zoom
       
  // constructeurs
  Soap_realisation(){};

  Soap_realisation(Soap3D_camera * cam);
  
  // cration de la nurbs d' angles de vue
  void creerNurbsAnglesVue();

  // cration de la nurbs du zoom
  void creerNurbsZoom();

  // cration des nurbs associ  une ralisation
  void creerNurbsRealisation();

  // obtenir l'angle theta en fonction du temps
  float camera_theta(float tps);

  // obtenir l'angle phy en fonction du temps
  float camera_phy(float tps);

  // obtenir le zoom (ouverture sur les x) en fontion du temps  
  float camera_zoom(float tps);

  //obtenir la nurbs sur l'angle theta
  Interpolation_NURBS getAngleTheta(){return angle_theta;};
  
  //obtenir la nurbs sur l'angle phy
  Interpolation_NURBS getAnglePhy(){return angle_phy;};
  
  //obtenir la nurbs sur le zoom
  Interpolation_NURBS getZoom(){return zoom;};
                   
};




