
/*
 *                        classe Soap_trajectoire
 *                        -------------------                         
 *                                                                    
 * C'est dans classe que sont concentres les informations relatives au dplacment 
 * d'un personnage 3D donn.
 * Une Soap_trajectoire est compose :                                    
 *                   - d'un personnage 3D associ a sa trajectoire
 *                   - d'une interpolation de la vitesse d'un personnage
 *                   - d'une interpolation de les angles du corps d'un personnage
 *                   - d'une interpolation de les angles de la tete d'un personnage
 *
 *
 */ 


#include"Soap_trajectoire.h"

//
// -> convertit_point(Soap3D_pointpassage* pt)
//
// Convertit un point de passage 3D en un point Point3Df utilisables pour la nurbs
// associe  la trajectoire 
//            - un Soap3D_pointpassage
//  <-son quivalent Point3Df
//

Point3Df Soap_trajectoire::convertit_point(Soap3D_pointpassage* pt){
      return Point3Df(pt->position->x,pt->position->y,pt->position->z);                         
      };
   
      
         
//
// ->convertit_tableau_point(vector<Soap3D_pointpassage *> pt_passage)
//
// Convertit un tableau de point de passage 3D en un tableau point de Point3Df utilisables pour la nurbs
// associe  la trajectoire 
//            - le tableau de point de passage Soap3D_pointpassage
//  <-son quivalent Vector_Point3Df
//      
      
Vector_Point3Df Soap_trajectoire::convertit_tableau_point(vector<Soap3D_pointpassage *> pt_passage){
 
 
		   Soap3D_pointpassage * temp;//variable temporaire sur un point 3D
                   Vector_Point3Df tab_points(pt_passage.size());
		   int j=0;//indice du nombre de points stop rencontrs

                   for(int i=0;i<pt_passage.size();i++){
		   temp=pt_passage[i];
                   if(temp->allure==stop) {j++;tab_points.resize(pt_passage.size()-j);}
		   else{
		                     
			   Point3Df ptres = convertit_point(temp);
			   tab_points[i-j] = ptres ;
			   }
                         }

                   return tab_points;                        
       };

//
// Constructeur
// ------------
// -> Soap_trajectoire(Soap3D_personnage * pers)
//
// Construit une instance d'une Soap_trajectoire,  partir d'un personnage
// Appel du super constructeur Interpolation_NURSB avec la liste des points de passage 3D
// qui lui sont associs
//

Soap_trajectoire::Soap_trajectoire(Soap3D_personnage * pers):Interpolation_NURBS(convertit_tableau_point(pers->pt_passage)){
           
      
        perso = pers;
	bool pt_stop=false;//a vrai si le point de passage prcdent  une allure stop
	float distance_ptprec=0.0;//distance du point prcdent, le point suivant un point stop est  la meme distance
	float distance_totale;//distance totale parcourue jusau'au point
  	int j=0;//indice du nombre de point stop rencontr

        for(int i=0;i<pers->pt_passage.size();i++){//pour chaque point de passage
	
	if(pt_stop) {//si le point de passage prcdent  une allure stop
	pers->pt_passage[i]->distance=distance_ptprec;pt_stop=false;//alors la distance parcourue du point courant est la meme que celle du point stop
	}
	
	else {//le point prcdent tait un point marche ou course
	if(pers->pt_passage[i]->allure==stop){//si le point courant est un point stop
		pt_stop=true;
		distance_totale=distance_pt1vspt2(pointsCles[0],pointsCles[i-j]);
		distance_ptprec=distance_totale;//le point suivant aurra la meme position,donc la meme distance parcourue
		perso->pt_passage[i]->distance = distance_totale;j++;
		}
	else {//le point courant est un point marche ou course
	
	    if(i==0) perso->pt_passage[i]->distance =0.0;//la distance parcourue au point initiale est nulle
	     
	    else {
	    //calcul de la distance totale parcourue jusqu'au point     
	        distance_totale = distance_pt1vspt2(pointsCles[0],pointsCles[i-j]);    

		//le point3D se trouve  une distance distance totale sur le parcours du peronnage
		perso->pt_passage[i]->distance = distance_totale;

	    }

	    }
	    }
	}//for
	
	//for(int i=0;i<perso->pt_passage.size();i++)cout<<" pt "<<i<<" dist parcourue :"<<perso->pt_passage[i]->distance<<endl;
	
	    creerNurbsVitesse();//cration de la nurbs vitesse

        }
	
//
// -> creerNurbsVitesse()
//
// Cration de la nurbs de la vitesse en suivant la distance parcourue et la vitesse
// en chaque points de passage
//

void Soap_trajectoire::creerNurbsVitesse(){


Vector_Point3Df pts(perso->pt_passage.size());//tableau des points cles
pts[0]=Point3Df(0.0,0.0,0.0);
Vector_Point3Df ptsderiv(perso->pt_passage.size());//tableau des points cles
float vitesse_en_i=0.0f;
int allure_i;

for(int i=1;i<perso->pt_passage.size();i++){
    
     allure_i =10* perso->pt_passage[i-1]->allure;//on prend l'allure du point prcdent

 
   
     pts[i]=Point3Df(perso->pt_passage[i]->distance,allure_i,0.0);
     ptsderiv[i]==Point3Df(0,0,1);

}

 NurbsCurvef courbe;
 courbe.globalInterpD(pts,ptsderiv,3,1);
 courbe.writePSp("courbe_derive.ps",pts,false,0,2);
 //vitesse= Interpolation_NURBS(pts);


}

//
// -> creerNurbsAngleCorps()
//
// Cration de la nurbs de l'angle du corps en suivant le temps et l'angle du corps
// en chaque points de passage
//


void Soap_trajectoire::creerNurbsAngleCorps(){
  
 Vector_Point3Df pts(perso->pt_passage.size());//tableau des points cles
 float tps;
 
 for(int i=0;i<perso->pt_passage.size();i++){
  tps=perso->pt_passage[i]->temps_arrivee;//rcupration du temps d'arrivee
  pts[i]=Point3Df(tps,perso->pt_passage[i]->angle_corps,0.0);
 
 }
 

 anglecorps= Interpolation_NURBS(pts);
 
}

//
// -> creerNurbsAngleTete()
//
// Cration de la nurbs de l'angle de la tete en suivant le temps et l'angle de la tete
// en chaque points de passage
//


void Soap_trajectoire::creerNurbsAngleTete(){
  
 Vector_Point3Df pts(perso->pt_passage.size());
 float tps,tpsd;
 int j=0;
 
 for(int i=0;i<perso->pt_passage.size();i++){
  tps=perso->pt_passage[i]->temps_arrivee;//rcupration du temps d'arrivee
  tpsd=perso->pt_passage[i]->temps_depart;//rcupration du temps de depart
  pts[i+j]=Point3Df(tps,perso->pt_passage[i]->angle_tete,0.0);
  if((int)tps!=(int)tpsd){j++;pts.resize(perso->pt_passage.size()+j);pts[i+j]=Point3Df(tpsd,perso->pt_passage[i]->angle_tete,0.0);};

 }
 
 angletete= Interpolation_NURBS(pts);
 
 
}

//
// -> tempsDistance(double tps, double vitesse)
//
// Obtenir la distance  partir de la vitesse et du temps
//            - le temps
//            - la vitesse
//   <- la distance
//
    
double Soap_trajectoire::tempsDistance(double tps, double vitesse){
        return tps * vitesse;
      }
      

//
// -> recherchePointPlusProche(float tps)
//
// Obtenir l'indice du point de passage dont le temps d'arrivee maximal  est infrieur au temps
//            - le temps
//   <- l'indice
//
    
  
int Soap_trajectoire::recherchePointPlusProche(float tps){
             
             int i =0;
             float temps = 0.f;
             while((tps>=temps)&&(i<perso->pt_passage.size())){
              temps=perso->pt_passage[i]->temps_arrivee;
              i++;                                              
             }
             return i-2;//l'indice retourn est celui du point infrieur
           }
     
//
// ->  Soap_trajectoire::temps_vs_u(vector<float> tableau_vitesse, float pasv, float tps, float u_min)
//
// Obtenir le parametre u de la nurbs vitesse suivant  le temps
//            - le tableau de correspondance entre u et le temps
//            - le pas de remplissage du tableau
//            - le temps restant
//            - le u_min du point de passage le plus proche
//   <- le parametre u
//
      
float Soap_trajectoire::temps_vs_u(vector<float> tableau_vitesse, float pasv, float tps, float u_min){
   float res,sup,inf;
   int i=0;
   

   while((i<tableau_vitesse.size())&&(tps>tableau_vitesse[i])) i++;
   sup = tableau_vitesse[i];
   inf = tableau_vitesse[i-1];
   res = u_min + (i-1)*pasv +(tps-inf)*pasv/(sup-inf);//cout<<"tps="<<tps<<" (tps-inf)*pasv/(sup-inf)="<<(tps-inf)*pasv/(sup-inf)<<" sup="<<sup<<" inf="<<inf<<"val i"<<i<<endl;
   return res;

}


//
// -> calcul_distance_parcourue(float tpt1,float tpt2,float tps,, float u_min, float u_max)
//
// Obtenir la distance restant  parcourir suivant  le temps restant
//            - le temps au point infrieur
//            - le temps au point suprieur
//            - le temps restant
//            - le u_min du point de passage infrieur
//            - le u_max du point de passage suprieur
//   <- la distance parcourue
//
         
float Soap_trajectoire::calcul_distance_parcourue(float tpt1,float tpt2,float tps, float u_min, float u_max){
    
    float u_tps = u_min + (u_max - u_min)*(tps)/(tpt2-tpt1);
    if(u_tps>u_max)u_tps=u_max;
        
    //on renvoit l'abscisse du point de la nurbs de degr 2 qui correspond  la distance parcourue
    return Cp(u_tps,vitesse.getCourbe()).x();
    
 
}

//
// -> position (float tps)
//
// Obtenir la position du personnage en fonction du temps restant
// On rcupre le point de passage possedant letemps d'arrivee le plus proche de tps
// On ralise ensuite l'interpolation (vitesse,temps) pour obtenir la distance parcourue
// puis le temps
//            - le temps du film
//   <- le point3D de la position
//     
Soap_point3D Soap_trajectoire::position (float tps) {
          int indice = recherchePointPlusProche(tps);
	  Soap_point3D res(0.f,0.f,0.f);
	  //if(tps<=tmp->getTpsD()) return le point3D correspondant suivant l'indice ?

          float tpsD = perso->pt_passage[indice]->temps_depart;
	  
	  if(tps<=tpsD) {
	  res.x=perso->pt_passage[indice]->position->x;
	  res.y=perso->pt_passage[indice]->position->y;
	  res.z=perso->pt_passage[indice]->position->z;
	  }
	  else{

	  
	  float tps_restant=tps - tpsD;
	  float dpt1 = perso->pt_passage[indice]->distance;
	  float dpt2 = perso->pt_passage[indice+1]->distance;
	  
	  float tpsA = perso->pt_passage[indice+1]->temps_arrivee;
	  
	  float umin=vitesse.obtient_u_vs_absisse(dpt1);
	  float umax=vitesse.obtient_u_vs_absisse(dpt2);
	  

	  
	  float dist = calcul_distance_parcourue(tpsD,tpsA,tps_restant,umin,umax);
	  float distance_parcourue=dist;
	  
	  //if(indice==0) distance_parcourue+=dist;
	  //else distance_parcourue-=dist;
	  
	  cout<<"++++++distance parcourue au temps t="<<tps<<" : "<<distance_parcourue<<endl;
	  
	  if(tps==0) {
	  res.x=perso->pt_passage[indice]->position->x;
	  res.y=perso->pt_passage[indice]->position->y;
	  res.z=perso->pt_passage[indice]->position->z;  
	  }
	  else {

	  Point3Df pt(curveDistance(distance_parcourue));
	  res.x=pt.x();
	  res.y=pt.y();
	  res.z=pt.z();
	  
	  }
	  
	  }
	  return res;
	  
   
       
}
	 
	 
//
// -> calcul_vitesse(float dist)
// Obtenir la vitesse suivant la distance
//            - la distance
//   <- la vitesse exact au point a distance dist
//
      
	 
double Soap_trajectoire::calcul_vitesse(float dist){

	return vitesse.obtient_ordonnee(dist);
	//il suffit de calculer l'ordonne de la nurbs 2D (distance,vitesse)

}


//
// ->  temps_entre(int pt1,int pt2)
//
// Obtenir le temps entre deux points
//            - indice du point 1
//            - indice du point 2
//   <- le temps calcul
//
      
double Soap_trajectoire::temps_entre(int pt1,int pt2){

  double vitesse=0.0;
  double temps=0.0;
  double d_pas=1.0;
  double dmin=0.0;
  double distpt1 = perso->pt_passage[pt1]->distance;
  double distpt2 = perso->pt_passage[pt2]->distance;
  double dmax=distpt2 - distpt1; 

  if(perso->pt_passage[pt1]->allure==stop){//si le point pt1 est un point stop
  
  temps=perso->pt_passage[pt1]->tempsattente;
  }
  else{
  for(double d = dmin+d_pas;d<=dmax;d+=d_pas){
    vitesse=calcul_vitesse(d+distpt1);
    if(vitesse>0)temps+=d_pas/vitesse;//la nurbs peut passer sous la barre des 0 !
    }
  }
    
 return temps;

}


//
// -> angle_corps(float temps)
//
// Obtenir l'angle du corps suivant le temps
//            - le temps
//   <- l'angle calcul
//
      
float Soap_trajectoire::angle_corps(float temps){

	return anglecorps.obtient_ordonnee(temps);
	//il suffit de calculer l'ordonne de la nurbs 2D (temps,angle corps)
}

//
// -> angle_tete(float temps)
//
// Obtenir l'angle de la tete suivant le temps
//            - le temps
//   <- l'angle calcul
//
     
float Soap_trajectoire::angle_tete(float temps){

	return angletete.obtient_ordonnee(temps);
	//il suffit de calculer l'ordonne de la nurbs 2D (temps,angletete)
}

