#define SOAP_TRAJECTOIRE_H

/*
 *                        classe Soap_trajectoire
 *                        -------------------                         
 *                                                                    
 * C'est dans classe que sont concentres les informations relatives au dplacment 
 * d'un personnage 3D donn.
 * Une Soap_trajectoire est compose :                                    
 *                   - d'un personnage 3D associ a sa trajectoire
 *                   - d'une interpolation de la vitesse d'un personnage
 *                   - d'une interpolation de les angles du corps d'un personnage
 *                   - d'une interpolation de les angles de la tete d'un personnage
 *
 *
 */ 
#ifndef INTERPOLATION_NURBS_H
#include "Interpolation/Interpolation_NURBS.h"
#endif
#ifndef SOAP3D_PERSONNAGE_H
#include "Soap3D_personnage.h"
#endif
#ifndef SOAP_VARIABLES_H
#include "../SoapGeneral/Soap_variables.h"
#endif
#ifndef SOAP_POINT3D_H
#include "../SoapGeneral/Soap_point3D.h"
#endif
#include <vector>


class Soap_trajectoire : public Interpolation_NURBS {//une Soap_trajectoire hrite directement de Interpolation_NURBS
      
public :
  
                /* ATTRIBUTS */

  Soap3D_personnage * perso;//pointeur vers un personnage 3D
                
  Interpolation_NURBS  vitesse;//interpolation sur la vitesse du personnage

  Interpolation_NURBS anglecorps;//interpolation sur l'angle du corps d'un personnage

  Interpolation_NURBS angletete;//interpolation sur l'angle de la tete d'un personnage
  
  
                /* METHODES */

  // constructeur
  Soap_trajectoire(Soap3D_personnage * pers);
  
  // convertit un point de passage 3D en un point Point3Df utilisables pour la nurbs associe  la trajectoire 
  Point3Df convertit_point(Soap3D_pointpassage *pt);
  
  // convertit un tableau de point de passage 3D en son quivalent pour la nurbs
  Vector_Point3Df convertit_tableau_point(vector<Soap3D_pointpassage *> pt_passage);                
 
  //obtenir la distance selon le temps du film et la vitesse     
  double tempsDistance(double tps, double vitesse);

  //recherche l'indice du point de passage d'un personnage le plus proche du temps indiqu
  int recherchePointPlusProche(float tps);

  //indique la position 3D du personnage selon le temps
  Soap_point3D position (float tps);

  //temps mis entre les points d'indice pt1 et pt2
  double temps_entre(int pt1,int pt2);

  //calcul la vitesse suivant la distance ou est rendu le personnage  
  double calcul_vitesse(float dist);
  
  //cration de la nurbs de vitesse
  void creerNurbsVitesse();

  //cration de la nurbs des angles du corps  
  void creerNurbsAngleCorps();

  //cration de la nurbs des angles de la tete
  void creerNurbsAngleTete();
  
  //obtenir la nurbs de la vitesse
  Interpolation_NURBS getCourbeVitesse(){return vitesse;};
  
  //obtenir la nurbs de l'ange du corps  
  Interpolation_NURBS getCourbeAngleCorps(){return anglecorps;};

  //obtenir la nurbs de l'ange de la tete  
  Interpolation_NURBS getCourbeAngleTete(){return angletete;};

  //obtenir la position du corps en fonction du temps
  float angle_corps(float temps);
  
  //obtenir la position de la tete en fonction du temps
  float angle_tete(float temps);

  //obtenir la valeur du parametre u en fonction du temps
  float temps_vs_u(vector<float> tableau_vitesse, float pasv, float tps_restant,float u_min);

  //calcul la distance parcourue entre le temps de passage point le plus proche et le temps totale
  float calcul_distance_parcourue(float tpsD,float tpsA,float tps_restant,float u_min, float u_max);
                   
};


