/***************************************************************************
      SOAP - Modlisation de scnes 3D  partir de spcifications en 2D
      -----------------------------------------------------------------

                               pov_camera.cpp
                                    --
                                description
                             -------------------
    Date          : Mardi 15 Mars 2005
    Auteur(s)     : BG
    Email         :
 ***************************************************************************/


#include "pov_camera.h"



/*
 * Constructeur POV_camera()
 *
 * Contructeur par dfaut qui cr une nouvelle camra en fixant le ratio 
 * 1.33:1, la position en (0,0,0) et ouverture en X  90. De plus, les angles
 * theta et phi sont fixs  0.
 *
 */
POV_camera::POV_camera()
{
    ratio = 1.33;
    location = new Coord3D();
    phi = 0;
    theta = 0;
    ouv_x = 90;
    ouv_y = 0;
}




/*
 * Constructeur POV_camera(float ratio_cam, Coord3D * position, float _phi, float _theta, float _ouv_x, float _ouv_y)
 *
 * Cr une nouvelle camra avec tous les lments passs en paramtres
 *
 */
POV_camera::POV_camera(float ratio_cam, Coord3D * position, float _phi, float _theta, float _ouv_x, float _ouv_y)
{
    ratio = ratio_cam;
    location = position;
    phi = _phi;
    theta = _theta;
    ouv_x = _ouv_x;
    ouv_y = _ouv_y; 
}



/*
 * Constructeur POV_camera(float ratio_cam, float _phi, float _theta, float _ouv_x, float _ouv_y)
 *
 * Idem prcdent sauf que la camra est place par dfaut en (0,0,0)
 *
 */
POV_camera::POV_camera(float ratio_cam, float _phi, float _theta, float _ouv_x, float _ouv_y)
{
    ratio = ratio_cam;
    location = new Coord3D();
    phi = _phi;
    theta = _theta;
    ouv_x = _ouv_x;
    ouv_y = _ouv_y; 
}




/*
 * Constructeur POV_camera(float ratio_cam)
 *
 * Idem prcdent sauf que tout est fix  part le ratio, ce qui permet de ne crer qu'une camra 
 * avec un ratio prdfini.
 *
 */
POV_camera::POV_camera(float ratio_cam)
{
    ratio = ratio_cam;
    location = new Coord3D();
    phi = 0;
    theta = 0;
    ouv_x = 90;
    ouv_y = 0;
}




/*
 * Destructeur ~POV_camera()
 *
 */
POV_camera::~POV_camera()
{
    delete location;
}



/*
 * string POV_camera::POV_tostring()
 *
 * Cette fonction renvoie une chaine de caractres correspondant  la dfinition
 * sous POV-Ray de la camra.
 * Cette chaine a la structure suivante :
 *        camera {
 *            up <0.0, 1.0, 0.0>
 *            right <ratio, 0.0, 0.0>
 *            location <pos_x, pos-y, pos_z>
 *            angle ouverture_en_x
 *            rotate <0, phi, 0>
 *            rotate <theta, 0, 0>
 *        }
 *
 */
string POV_camera::POV_tostring()
{
    string s = "camera {up <0.0, 1.0, 0.0> right <"+FloatVersChaine(ratio)+", 0.0, 0.0>";
    s+="location <"+FloatVersChaine(location->get_x())+","+FloatVersChaine(location->get_y())+","+FloatVersChaine(location->get_z())+">";
    s+=" angle "+FloatVersChaine(ouv_x);
    s+=" rotate <0,"+FloatVersChaine(phi)+",0>";
    s+=" rotate <"+FloatVersChaine(theta)+",0,0>}";

    return s;
}  

