/***************************************************************************
      SOAP - Modlisation de scnes 3D  partir de spcifications en 2D
      -----------------------------------------------------------------

                                pov_fichier.cpp
                                    --
                                description
                             -------------------
    Date          : Lundi 14 Mars 2005
    Auteur(s)     : BG
    Email         :
 ***************************************************************************/

#include <iostream> 
#include <fstream>
#include <string>
using namespace std;
#include "pov_fichier.h"

POV_fichier::POV_fichier(string name, string version_pov, POV_camera * cam, POV_ciel * sky, POV_plan * plane)
{
    nom = name;
    version = version_pov;
    camera = cam;
    ciel = sky;
    plan = plane;
}

POV_fichier::~POV_fichier()
{
    delete ciel;
    delete camera;
    delete plan;
}

void POV_fichier::POV_genere()
{
    string nomfichier = "./pov_files/"+nom;
    nomfichier += ".pov";

    // Ouverture du fichier
    ofstream fichierPOV;
    fichierPOV.open(nomfichier.c_str(), ios_base::out);

    // Teste si le fichier a pu etre ouvert ou non
    if(!fichierPOV)
    {
        cout<<"Impossible d'ouvrir le fichier "<<nomfichier<<endl;
        exit(0);
    }

    // Debut d'ecriture dans le fichier : Toutes les informations permettant d'ecrire dans le fichier
    // POV sont renseignees ici.

    // Tout d'abord on ecrit la version de POV utilise
    fichierPOV<<"#version "<<version<<endl;
    // On ecrit ensuite la valeur du gamma
    fichierPOV<<"global_settings { assumed_gamma 2.2 }"<<endl;
    // Liste des fichiers include utilises
    fichierPOV<<"#include "<<'"'<<"colors.inc"<<'"'<<endl;
    fichierPOV<<"#include "<<'"'<<"textures.inc"<<'"'<<endl;
    fichierPOV<<"#include "<<'"'<<"skies.inc"<<'"'<<endl;
    fichierPOV<<"#include "<<'"'<<"bonhomme.inc"<<'"'<<endl;

    // Camera
    fichierPOV<<camera->POV_tostring()<<endl;

    // Ciel
    fichierPOV<<ciel->POV_tostring()<<endl;

    // Lumieres
    for(int i=0;i<liste_lumieres.size();i++)
    {
        fichierPOV<<liste_lumieres[i]->POV_tostring()<<endl;
    } 

    // Plan                        
    fichierPOV<<plan->POV_tostring()<<endl;

    // Personnages
    for(int i=0;i<liste_personnages.size();i++)
    {
        fichierPOV<<liste_personnages[i]->POV_tostring()<<endl;
    } 
        
    //fichierPOV<<lumiere->POV_tostring()<<endl;
    // Fermeture du fichier
    fichierPOV.close();
}


void POV_fichier::ajoute_lumiere(POV_lumiere * light)
{
    // Ajoute la lumiere dans la liste
    liste_lumieres.push_back(light);
}

void POV_fichier::ajoute_personnage(POV_personnage * perso)
{
    // Ajoute le personnage dans la liste de tous les personnages
    liste_personnages.push_back(perso);
}



